#' Fried Frailty Phenotype Dataset
#'
#' A dataset containing variables used to compute Fried frailty phenotype measures.
#' The exact variables/columns depend on your project definitions.
#'
#' @format A data frame.
#' @name fried_frailty
#' @keywords datasets
NULL
