#' Merged Frailty Index Dataset (49-items and fried frailty)
#'
#' A dataset containing variables used to compute a 49-item frailty index (FI) and Fried frailty.
#' The exact variables/columns depend on your project definitions.
#'
#' @format A data frame.
#' @name merged_data
#' @keywords datasets
NULL
