#' Frailty Index Dataset (49-items)
#'
#' A dataset containing variables used to compute a 49-item frailty index (FI).
#' The exact variables/columns depend on your project definitions.
#'
#' @format A data frame.
#' @name fi_49item
#' @keywords datasets
NULL
